// Copyright 2011, Google Inc. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package com.google.api.ads.dfa.lib.client;

import com.google.api.ads.common.lib.client.AdsServiceClient;
import com.google.api.ads.common.lib.exception.ServiceException;
import com.google.api.ads.common.lib.soap.SoapClientHandlerInterface;
import com.google.api.ads.common.lib.utils.logging.AdsServiceLoggers;
import com.google.inject.Inject;
import com.google.inject.assistedinject.Assisted;

/**
 * Wrapper of underlying SOAP client which allows access for setting
 * headers retrieved from the session.
 *
 * @author api.arogal@gmail.com (Adam Rogal)
 * @author api.jdilallo@gmail.com (Joseph DiLallo)
 */
public class DfaServiceClient extends AdsServiceClient<DfaServiceSession, DfaServiceDescriptor> {

  /**
   * Constructor.
   *
   * @param soapClient the SOAP client
   * @param dfaServiceDescriptor the DFA service descriptor
   * @param dfaServiceSession the DFA service session
   * @param soapClientHandler the SOAP client handler
   * @param dfaHeaderHandler the DFA header handler
   * @param adsServiceLoggers the ads service loggers
   */
  @Inject
  public DfaServiceClient(@Assisted("soapClient") Object soapClient,
      @Assisted("adsServiceDescriptor") DfaServiceDescriptor dfaServiceDescriptor,
      @Assisted("adsServiceSession") DfaServiceSession dfaServiceSession,
      SoapClientHandlerInterface<Object> soapClientHandler,
      DfaHeaderHandler dfaHeaderHandler,
      AdsServiceLoggers adsServiceLoggers)
      throws ServiceException {
    super(soapClient, dfaServiceSession, dfaServiceDescriptor, soapClientHandler, dfaHeaderHandler,
        adsServiceLoggers);
  }
}
