// Copyright 2011, Google Inc. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package com.google.api.ads.dfa.lib.factory;

import com.google.api.ads.common.lib.conf.AdsApiConfiguration;
import com.google.api.ads.common.lib.conf.AdsLibConfiguration;
import com.google.api.ads.common.lib.factory.AdsServiceClientFactory;
import com.google.api.ads.common.lib.factory.BaseAdsServiceClientFactory;
import com.google.api.ads.dfa.lib.DfaModule;
import com.google.api.ads.dfa.lib.client.DfaServiceClient;
import com.google.api.ads.dfa.lib.client.DfaServiceDescriptor;
import com.google.api.ads.dfa.lib.client.DfaServiceSession;
import com.google.api.ads.dfa.lib.client.DfaUser;
import com.google.common.annotations.VisibleForTesting;
import com.google.inject.AbstractModule;
import com.google.inject.Guice;
import com.google.inject.Inject;
import com.google.inject.Injector;

import java.net.MalformedURLException;

/**
 * Factory which creates DFA service clients.
 *
 * @author api.jdilallo@gmail.com (Joseph DiLallo)
 */
public class DfaServiceClientFactory extends BaseAdsServiceClientFactory<DfaServiceClient,
                                                                         DfaServiceSession,
                                                                         DfaServiceDescriptor,
                                                                         DfaUser> {

  final static Injector injector = Guice.createInjector(new DfaModule());

  /**
   * Default constructor.
   */
  public DfaServiceClientFactory() {
    super(injector);
  }

  /**
   * Constructor.
   *
   * @param module the module used for binding.
   */
  @VisibleForTesting
  DfaServiceClientFactory(AbstractModule module) {
    super(module);
  }

  /**
   * Constructor which also sets the endpoint server.
   *
   * @param endpointServer the server hosting the DFA web services
   * @throws MalformedURLException if the endpoint server was malformed
   */
  @SuppressWarnings("unchecked") // Type is checked.
  public DfaServiceClientFactory(String endpointServer) throws MalformedURLException {
    super(injector);
    this.setEndpointServer(endpointServer);
  }

  /**
   * Constructor used for injection.
   *
   * @param adsServiceClientFactory the ads service client factory
   * @param libConfiguration the lib configuration
   * @param apiConfiguration the API configuration
   */
  @Inject
  @VisibleForTesting
  DfaServiceClientFactory(AdsServiceClientFactory<DfaServiceClient,
                                                  DfaServiceSession,
                                                  DfaServiceDescriptor,
                                                  DfaUser>
                          adsServiceClientFactory, AdsLibConfiguration libConfiguration,
                          AdsApiConfiguration apiConfiguration) {
    super(adsServiceClientFactory, libConfiguration, apiConfiguration);
  }
}
